# flyweight tag name
# $Id: tagname.rb,v 1.1 2001/07/29 03:00:23 moro Exp $

class TagName
    SIZE_TAGS = [
	'0','1','2','3','4','5','6','7','8','9'
    ].freeze
    COLOR_TAGS = [
	'r','g','b','c','m','y','k','w'
    ].freeze
    TAG_NAMES = [
	'B','S','I','U',
	'EM','PL','TT',
    ] + SIZE_TAGS + COLOR_TAGS
    TAG_NAMES.freeze

    def initialize(name)
	@name = name.intern
	@name_str = name
	@name_str.freeze
	@open_tag_str = "<#{name}>".freeze
	@close_tag_str = "</#{name}>".freeze
	@length = name.length

	@color = if COLOR_TAGS.include? name
		     name[0]
		 else
		     nil
		 end
	@size  = if SIZE_TAGS.include? name
		     name.to_i
		 else
		     nil
		 end
    end
    attr_reader :name, :open_tag_str, :close_tag_str, :length, :color, :size
    
    def to_str
	@name_str
    end

    def hash
	@name.hash
    end

    def eql?(other)
	self == other
    end

    def ==(other)
	self.equal? other	# assume flyweight
    end

    Str2TagName = {}
    SizeTags = []
    ColorTags = {}

    module Constants
    end
    
    TAG_NAMES.each do |name|
	tag = TagName.new(name).freeze
	Str2TagName[name] = tag
	Str2TagName[name.intern] = tag
	Constants.const_set("TAG_#{name}", tag)

	if tag.size
	    SizeTags[tag.size] = tag
	elsif tag.color
	    ColorTags[tag.color] = tag
	end
    end
    SizeTags.freeze
    ColorTags.freeze

    def self.sizetag(size)
	SizeTags[size]
    end

    def self.colortag(color)
	ColorTags[color]
    end
    
    def self.get(name)
	Str2TagName[name]
    end
    
    class <<self
	private :new
    end
end

if __FILE__ == $0
    (TagName::TAG_NAMES).each do |name|
	print "#{name} => "
	p TagName.get(name)
    end

    print "\n"

    (TagName::Constants).constants.each do |name|
	if /^TAG_/ =~ name
	    print "#{name} => "
	    p ((TagName::Constants).const_get(name))
	end
    end

    print "\n"

    p TagName.get('EM') == TagName::Constants::TAG_EM # => true

    p TagName.get(:EM) == TagName::Constants::TAG_EM # => true

    p TagName.new('EM')		# => error
end

include TagName::Constants
